<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: DateUtil.class.php 24738 2008-10-20 08:49:32Z rgasch $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @uses generic data utililty class
 * @package Zikula_Core
 */

/**
 * constants for this class
 */
define ('DATEFORMAT_FIXED', '%Y-%m-%d %H:%M:%S');

/**
 * DateUtil
 *
 * @package Zikula_Core
 * @subpackage DateUtil
 */
class DateUtil
{
    /**
     * Return a formatted datetime for the given timestamp (or for now)
     *
     * @param time      The timestamp (string) which we wish to format (default==now)
     * @param format    The format to use when formatting the date (optional)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function getDatetime ($time='', $format=DATEFORMAT_FIXED)
    {
        switch(trim(strtolower($format))){
            case 'datelong':
                $format = _DATELONG;
                break;
            case 'datebrief':
                $format = _DATEBRIEF;
                break;
            case 'datestring':
                $format = _DATESTRING;
                break;
            case 'datestring2':
                $format = _DATESTRING2;
                break;
            case 'datetimebrief':
                $format = _DATETIMEBRIEF;
                break;
            case 'datetimelong':
                $format = _DATETIMELONG;
                break;
            case 'timebrief':
                $format = _TIMEBRIEF;
                break;
            case 'timelong':
                $format = _TIMELONG;
                break;
            default:
        } // switch

        // bp 2008-03-29 - translation of day and month names if Windows utf-8 system is used
        if ($time) {
            $dtstr = strftime($format, $time);
        } else {
            $dtstr = strftime($format);
        }
        $encoding = pnSessionGetVar('encoding');
        $cs = strtolower(_CHARSET);
        $cs == 'utf-8' ? $cs = 'utf8' : '';	
        // applies only if $encoding is set, iconv is available and we're using utf8
        if ((!empty($encoding)) && ($cs == 'utf8')) {
            // check for iconv - not included in std. win distribution
            if (!extension_loaded('iconv')) {
                if (!dl('iconv.so')) {
                    return $dtstr;
                }
            }	
            if ($encoding !=  $cs) {
                $dtstr = iconv($encoding, $cs, $dtstr);
            }
        }
        return $dtstr;
    }


    /**
     * Reformat a given datetime according to the specified format
     *
     * @param datetime    The (string) datetime to reformat
     * @param format      The format to use when formatting the date (optional)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function formatDatetime ($datetime=null, $format=DATEFORMAT_FIXED)
    {
        if ($datetime === null) {
            return '';
        }

        if (!$datetime) {
            return strftime($format);
        }
        
        $dTime = DateUtil::makeTimestamp($datetime);
        return DateUtil::getDatetime ($dTime, $format);
    }


    /**
     * Build a datetime string from the supplied fields
     *
     * @param year      The year
     * @param month     The month
     * @param day       The day
     * @param hour      The hour (optional) (default==0)
     * @param minute    The minute (optional) (default==0)
     * @param second    The second (optional) (default==0)
     * @param format    The format to use when formatting the date (optional)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function buildDatetime ($year, $month, $day, $hour=0, $minute=0, $second=0, $format=DATEFORMAT_FIXED)
    {
        $dTime = mktime ($hour, $minute, $second, $month, $day, $year);
        return strftime($format, $dTime);
    }


    /**
     * Return a formatted datetime at the end of the business day n days from now
     *
     * @param days      The number of days to advance (optional) (detault==1)
     * @param format    The format to use when formatting the date (optional)
     * @param hour      The hour of the target time to set (optional) (default=null, means params is taken from now)
     * @param minute    The minute of the target time to set (optional) (default=null, means params is taken from now)
     * @param second    The second of the target time to set (optional) (default=null, means params is taken from now)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function getDatetime_NextDay ($num=1, $format=DATEFORMAT_FIXED, $year=null, $month=null, $day=null, $hour=null, $minute=null, $second=null)
    {
        $next = mktime ($hour!=null   ? (int)$hour     : date('H'),
                        $minute!=null ? (int)$minute   : date('i'),
                        $second!=null ? (int)$second   : date('s'),
                        $month!=null  ? (int)$month    : date('m'),
                        $day!=null    ? (int)$day+$num : date('d')+$num,
                        $year!=null   ? (int)$year     : date('y'));
        return strftime($format, $next);
    }


    /**
     * Return a formatted datetime at the end of the business day n week from now
     *
     * @param num       The number of weeks to advance (optional) (detault==1)
     * @param format    The format to use when formatting the date (optional)
     * @param year      The year of the target time to set (optional) (default=null, means param is taken from now)
     * @param month     The month of the target time to set (optional) (default=null, means param is taken from now)
     * @param day       The day of the target time to set (optional) (default=null, means param is taken from now)
     * @param hour      The hour of the target time to set (optional) (default=null, means param is taken from now)
     * @param minute    The minute of the target time to set (optional) (default=null, means param is taken from now)
     * @param second    The second of the target time to set (optional) (default=null, means param is taken from now)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function getDatetime_NextWeek ($num=1, $format=DATEFORMAT_FIXED, $year=null, $month=null, $day=null, $hour=null, $minute=null, $second=null)
    {
        $num *= 7;
        $next = mktime ($hour!=null   ? (int)$hour     : date('H'),
                        $minute!=null ? (int)$minute   : date('i'),
                        $second!=null ? (int)$second   : date('s'),
                        $month!=null  ? (int)$month    : date('m'),
                        $day!=null    ? (int)$day+$num : date('d')+$num,
                        $year!=null   ? (int)$year     : date('y'));
        return strftime($format, $next);
    }


    /**
     * Return a formatted datetime at the end of the business day n months from now
     *
     * @param num       The number of months to advance (optional) (default=1)
     * @param format    The format to use when formatting the date (optional)
     * @param year      The year of the target time to set (optional) (default=null, means param is taken from now)
     * @param month     The month of the target time to set (optional) (default=null, means param is taken from now)
     * @param day       The day of the target time to set (optional) (default=null, means param is taken from now)
     * @param hour      The hour of the target time to set (optional) (default=null, means params is taken from now)
     * @param minute    The minute of the target time to set (optional) (default=null, means params is taken from now)
     * @param second    The second of the target time to set (optional) (default=null, means params is taken from now)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function getDatetime_NextMonth ($num=1, $format=DATEFORMAT_FIXED, $year=null, $month=null, $day=null, $hour=null, $minute=null, $second=null)
    {
        $next = mktime ($hour!=null   ? (int)$hour     : date('H'),
                        $minute!=null ? (int)$minute   : date('i'),
                        $second!=null ? (int)$second   : date('s'),
                        $month!=null  ? (int)$month+$num : date('m')+$num,
                        $day!=null    ? (int)$day      : date('d'),
                        $year!=null   ? (int)$year     : date('y'));
        return strftime($format, $next);
    }


    /**
     * Return a formatted datetime at the end of the business day n years from now
     *
     * @param num       The number of years to advance (optional) (default=1)
     * @param format    The format to use when formatting the date (optional)
     * @param year      The year of the target time to set (optional) (default=null, means param is taken from now)
     * @param month     The month of the target time to set (optional) (default=null, means param is taken from now)
     * @param day       The day of the target time to set (optional) (default=null, means param is taken from now)
     * @param hour      The hour of the target time to set (optional) (default=null, means params is taken from now)
     * @param minute    The minute of the target time to set (optional) (default=null, means params is taken from now)
     * @param second    The second of the target time to set (optional) (default=null, means params is taken from now)
     *
     * @return datetime The datetime formatted according to the specified format
     */
    function getDatetime_NextYear ($num=1, $format=DATEFORMAT_FIXED, $year=null, $month=null, $day=null, $hour=null, $minute=null, $second=null)
    {
        $next = mktime ($hour!=null   ? (int)$hour     : date('H'),
                        $minute!=null ? (int)$minute   : date('i'),
                        $second!=null ? (int)$second   : date('s'),
                        $month!=null  ? (int)$month    : date('m'),
                        $day!=null    ? (int)$day      : date('d'),
                        $year!=null   ? (int)$year+$num : date('y')+$num);
        return strftime($format, $next);
    }


    /**
     * Return the date portion of a datetime timestamp
     *
     * @param datetime  The date to parse (optional) (default=='', reverts to now)
     * @param format    The format to use when formatting the date (optional)

     * @return string   The Date portion of the specified datetime
     */
    function getDatetime_Date ($datetime='', $format=DATEFORMAT_FIXED)
    {
        if (!$datetime) {
            $datetime = DateUtil::getDatetime ();
        }

        $dTime = strtotime($datetime);
        $sTime = DateUtil::getDatetime ($dTime, $format);

        if ($format == DATEFORMAT_FIXED) {
            return substr ($sTime, 0, 10);
        }

        $spaceOffset = strpos ($datetime, ' ');
        return substr ($sTime, 0, $spaceOffset);
    }


    /**
     * Return the time portion of a datetime timestamp
     *
     * @param datetime  The date to parse (optional) (default=='', reverts to now)
     * @param format    The format to use when formatting the date (optional)
     *
     * @return string   The Time portion of the specified datetime
     */
    function getDatetime_Time ($datetime='', $format=DATEFORMAT_FIXED)
    {
        if (!$datetime) {
            $datetime = DateUtil::getDatetime ();
        }

        $dTime = strtotime($datetime);
        $sTime = DateUtil::getDatetime ($dTime, $format);

        if ($format == DATEFORMAT_FIXED) {
            return substr ($sTime, 11);
        }

        $spaceOffset = strpos ($datetime, ' ');
        return substr ($datetime, $spaceOffset+1);
    }


    /**
     * Return the requested field from the supplied date
     *
     * Since the date fields can change depending on the date format,
     * the following convention is used when referring to date fields:<br />
     *   Field 1    ->     Year<br />
     *   Field 2    ->     Month<br />
     *   Field 3    ->     Day<br />
     *   Field 4    ->     Hour<br />
     *   Field 5    ->     Minute<br />
     *   Field 6    ->     Second<br />
     *
     *
     * @param datetime  The field number to return
     * @param field     The date to parse (default=='', reverts to now)
     *
     * @return string   The requested datetime field
     */
    function getDatetime_Field ($datetime, $field)
    {
        if (!$datetime) {
            $datetime = DateUtil::getDatetime ();
        }

        $dTime = strtotime($datetime);
        $sTime = DateUtil::getDatetime ($dTime);

        $field--;         // adjust for human counting

        if ($field <= 2)  // looking for a date part
        {
            $date = DateUtil::getDatetime_Date ($sTime);
            $fields = explode ('-', $date);
            return $fields[$field];
        }
        else              // looking for a time part
        {
            $field -= 3;
            $time = DateUtil::getDatetime_Time ($sTime);
            $fields = explode (':', $time);
            return $fields[$field];
        }
    }


    /**
     * Return an structured array holding the differences between 2 dates.
     *
     * The returned array will be structured as follows:<br>
     *   Array (<br />
     *          [d] => _numeric_day_value_<br />
     *          [h] => _numeric_hour_value_<br />
     *          [m] => _numeric_minute_value_<br />
     *          [s] => _numeric_second_value_ )<br />
     *
     * @param date1     The first date
     * @param date2     The second date
     *
     * @return array    The structured array containing the datetime difference
     */
    function getDatetimeDiff ($date1, $date2)
    {
        $s = strtotime($date2) - strtotime($date1);

        $d = intval($s/86400);
        $s -= $d*86400;
        $h = intval($s/3600);
        $s -= $h*3600;
        $m = intval($s/60);
        $s -= $m*60;

        return array("d"=>$d,"h"=>$h,"m"=>$m,"s"=>$s);
    }


    /**
     * Return an field holding the differences between 2 dates expressed
     * in units of the field requested
     *
     * Since the date fields can change depending on the date format,
     * the following convention is used when referring to date fields:<br />
     *   Field 1    ->     Year<br />
     *   Field 2    ->     Month<br />
     *   Field 3    ->     Day<br />
     *   Field 4    ->     Hour<br />
     *   Field 5    ->     Minute<br />
     *   Field 6    ->     Second<br />
     *
     * @param date1     The first date
     * @param date2     The second date
     * @param field     The field (unit) in which we want the different (optional) (default=5)
     *
     * @return float The difference in units of the specified field
     */
    function getDatetimeDiff_AsField ($date1, $date2, $field=5)
    {
        $s    = strtotime($date2) - strtotime($date1);
        $diff = 0;

        if ($field == 1) {
            $diff = $s/(60*60*24*31*12);
        }
        else
        if ($field == 2) {
            $diff = $s/(60*60*24*31);
        }
        else
        if ($field == 3) {
            $diff = $s/(60*60*24);
        }
        else
        if ($field == 4) {
            $diff = $s/(60*60);
        }
        else
        if ($field == 5) {
            $diff = $s/(60);
        }
        else
        {
            $diff = $s;
        }

        return $diff;
    }


    /**
     * Calculate day-x of KW in a YEAR
     * @param day       0 for monday, 6 for sunday,....
     * @param kw        week of the year
     * @param year      year
     * @param flag      u or s (unixtimestamp or MySQLDate)
     *
     * @return unixtimestamp or sqlDate
     */
    function getDateofKW ($day, $kw, $year, $flag='s')
    {
        $wday = date('w',mktime(0,0,0,1,1,$year)); // 1=Monday,...,7 = Sunday

        if ($wday <= 4) {
            $firstday = mktime(0,0,0,1,1-($wday-1)+$day,$year);
        } else {
            $firstday = mktime(0,0,0,1,1+(7-$wday+1)+$day,$year);
        }

        $month = date('m',$firstday);
        $year  = date('Y',$firstday);
        $day   = date('d',$firstday);

        $adddays = ($kw-1)*7;

        if ($flag != 's') {
            $return = mktime(0,0,0,$month,$day+$adddays,$year);
        } else {
            $return = DateUtil::getDatetime(mktime(0,0,0,$month, $day+$adddays, $year));
        }

        return $return;
    }


    /**
     * Return a the number of days in the given month/year
     *
     * @param month     The (human) month number to check
     * @param year      The year number to check
     *
     * @return integer  The number of days in the given month/year
     */
    function getDaysInMonth ($month, $year)
    {
        if ($month < 1 || $month > 12) {
            return 0;
        }

        $days = array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

        $d = $days[$month-1];

        if ($month == 2) {
            // Check for leap year, no 4000 rule
            if ($year%4 == 0) {
                if ($year%100 == 0) {
                    if ($year%400 == 0) {
                        $d = 29;
                    }
                } else {
                    $d = 29;
                }
            }
        }

        return $d;
    }


    /**
     * Return an array of weekdays for the given month
     *
     * @param month     The (human) month number to check
     * @param year      The year number to check
     *
     * @return integer  The number of days in the given month/year
     */
    function getWeekdaysInMonth ($month, $year)
    {
        $nDays = DateUtil::getDaysInMonth ($month, $year);

        $weekdays = array();
        for ($i=1; $i<=$nDays; $i++) {
            $time = mktime (12, 0, 0, $month, $i, $year);
            $tDate = getdate ($time);
            $weekdays[$i] = $tDate['wday'];
        }

        return $weekdays;
    }


    /**
     * Return an array of dates for the given month
     *
     * @param month     The (human) month number to check
     * @param year      The year number to check
     *
     * @return integer  The number of days in the given month/year
     */
    function getMonthDates ($month, $year)
    {
        $dates = array();
        $days = DateUtil::getDaysInMonth ($month, $year);

        for ($i=1; $i<=$days; $i++) {
            $dates[$i] = DateUtil::buildDatetime ($year, $month, $i);
        }

        return $dates;
    }


    /**
     * Parses a user interface date string (excluding time) into a timestamp
     *
     * Currently only the format YYYY-MM-DD is supported, but future versions
     * will be able to parse more diverse formats.
     *
     * @param text      The UI date string
     *
     * @return date     The timestamp or null in case of errors
     */
    function parseUIDate($text)
    {
        $elements = split('-', $text);
        if (count($elements) == 3)
        {
            list ($year,$month,$day) = $elements;
        }
        else
        {
            $year = $month = $day = 0;
        }

        if (!checkdate($month,$day,$year))
        {
            return null;
        }

        return mktime(0,0,0, $month,$day,$year);
    }


    /**
     * Parses a user interface date+time string into a timestamp
     *
     * Currently only the format YYYY-MM-DD HH:MM is supported, but future versions
     * will be able to parse more diverse formats.
     *
     * @param text      The UI date+time string
     *
     * @return date     The timestamp or null in case of errors
     */
    function parseUIDateTime($text)
    {
        if (preg_match("/(\\d+)-(\\d+)-(\\d+)(\s+(\\d+)(:(\\d+))?)?/", $text, $matches))
        {
            $year = $matches[1];
            $month = $matches[2];
            $day = $matches[3];
            $sec = 0;

            if (count($matches) >= 4)
            {
                $hour = $matches[5];
            }
            else
            {
                $hour = 0;
            }

            if (count($matches) >= 5)
            {
                $min = $matches[7];
            }
            else
            {
                $min = 0;
            }

            if (!checkdate($month,$day,$year) || $hour>23 || $min>59)
            {
                return null;
            }
        }
        else
          return null;

        return mktime($hour,$min,$sec, $month,$day,$year);
    }

    /**
     * create a unix timestamp from either a unix timestamp (sic!), a MySQL timestamp
     * or a string. This code is taken from smarty_make_timestamp.php,
     * credits go to Monte Ohrt <monte at ohrt dot com>
     *
     * We use a copy of the code here due to performance reasons.
     *
     * @param int or text       a timestamp in one of the formats mentioned
     *
     * @return int              a unix timestamp
     *
     */
    function makeTimestamp($string='')
    {
        if(empty($string)) {
            // use "now":
            $time = time();
        
        } elseif (preg_match('/^\d{14}$/', $string)) {
            // it is mysql timestamp format of YYYYMMDDHHMMSS?            
            $time = mktime(substr($string, 8, 2),substr($string, 10, 2),substr($string, 12, 2),
                           substr($string, 4, 2),substr($string, 6, 2),substr($string, 0, 4));
            
        } elseif (is_numeric($string)) {
            // it is a numeric string, we handle it as timestamp
            $time = (int)$string;
            
        } else {
            // strtotime should handle it
            $time = strtotime($string);
            if ($time == -1 || $time === false) {
                // strtotime() was not able to parse $string, use "now":
                $time = time();
            }
        }
        return $time;
    }
}
