<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: FileUtil.class.php 24668 2008-09-19 17:33:40Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @package Zikula_Core
 */

/**
 * FileUtil
 *
 * @package Zikula_Core
 * @subpackage FileUtil
 */
class FileUtil
{
    /**
     * Given a filename (complete with path) get the file basename
     *
     * @param filename   The filename to process
     * @param keepDot    whether or not to return the dot with the basename
     *
     * @return string The file's filename
     */
    function getFilebase ($filename, $keepDot=false)
    {
        if (!$filename) {
            return pn_exit ('FileUtil::getFilename: filename is empty');
        }

        $base = basename ($filename);
        $p = strrpos ($base, '.');
        if ($p !== false) {
            if ($keepDot) {
                return substr($base, 0, $p+1);
            } else {
                return substr($base, 0, $p);
            }
        }

        return $filename;
    }


    /**
     * Get the basename of a filename
     *
     * @param filename     The filename to process
     *
     * @return string     The file's basename
     */
    function getBasename ($filename)
    {
      if (!$filename) {
          return pn_exit ('FileUtil::getBasename: filename is empty');
      }

      return basename ($filename);
    }


    /**
     * Get the file's extension
     *
     * @param filename    The filename to process
     * @param keepDot     whether or not to return the '.' with the extension
     *
     * @return string The file's extension
     */
    function getExtension ($filename, $keepDot=false)
    {
        if (!$filename) {
            return pn_exit ('FileUtil::getExtension: filename is empty');
        }

        $p = strrpos ($filename, '.');
        if ($p !== false) {
            if ($keepDot) {
                return substr($filename, $p);
            } else {
                return substr($filename, $p+1);
            }
        }

        return '';
    }


    /**
     * Strip the file's extension
     *
     * @param filename    The filename to process
     * @param keepDot     whether or not to return the '.' with the extension
     *
     * @return string The filename without the extension
     */
    function stripExtension ($filename, $keepDot=false)
    {
        if (!$filename) {
            return pn_exit ('FileUtil::getExtension: filename is empty');
        }

        $p = strrpos ($filename, '.');
        if ($p !== false) {
            if ($keepDot) {
                return substr($filename, 0, $p+1);
            } else {
                return substr($filename, 0, $p);
            }
        }

        return $filename;
    }


    /**
     * Generate a random filename
     *
     * @param min           Minimum number of characters
     * @param max           Maximum number of characters
     * @param useupper      whether to use uppercase characters
     * @param usenumbers    whether to use numeric characters
     * @param usespecial    whether to use special characters
     *
     * @return string The generated filename extension
     */
    function generateRandomFilename ($min, $max, $useupper=false, $usenumbers=true, $usespecial=false)
    {
        $rnd   = '';
        $chars = 'abcdefghijklmnopqrstuvwxyz';

        if ($useupper) {
            $chars .= 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        }

        if ($usenumbers) {
            $chars .= '0123456789';
        }

        if ($usespecial) {
            $chars .= '~@#$%^*()_+-={}|][';
        }

        $charlen = strlen($chars)-1;

        $len = mt_rand ($min, $max);
        for ($i = 0; $i < $len; $i++) {
            $rnd .= $chars[mt_rand(0, $charlen)];
        }

        return $rnd;
    }


    /**
     * Recursiveley generate a file listing
     *
     * @param rootPath      The root-path we wish to start at
     * @param recurse       whether or not to recurse directories (optional) (default=true)
     * @param relativePath  whether or not to list relative (vs abolute) paths (optional) (default=true)
     * @param extension     The file extension to scan for (optional) (default=null)
     * @param directories   Include directories in listing (optional) (default=true)
     *
     * @return boolean The return code from mkdir()
     */
    function getFiles ($rootPath, $recurse=true, $relativePath=true, $extension=null, $directories=true)
    {
        $files = array();

        if (!file_exists($rootPath) || !is_dir($rootPath) || !is_readable($rootPath)) {
            return $files;
        }

        $el = ($extension ? strlen($extension) : 0);
        $dh = opendir($rootPath);
        while(($file = readdir($dh)) !== false) {
            if ($file != '.' && $file != '..' && $file != 'CVS' && $file != '.svn' && $file != 'index.html') {
                $path = "$rootPath/$file";

                if (!$directories && is_dir($path)) {
                    continue;
                }

                if (!$extension || substr($file, -$el) == $extension) {
                    if ($relativePath) {
                        $files[] = $file;
                    } else {
                        $files[] = $path;
                    }
                }

                if ($recurse && is_dir($path)) {
                    $files = array_merge ((array)$files,
                                          (array)FileUtil::getFiles($path, $recurse, $relativePath, $extension));
                }
            }
        }

        closedir($dh);
        return $files;
    }


    /**
     * Recursiveley create a directory path
     *
     * @param path        The path we wish to generate
     * @param mode        The (UNIX) mode we wish to create the files with
     *
     * @return boolean The return code from mkdir()
     */
    function mkdirs ($path, $mode=null)
    {
        if (is_dir($path)) {
            return true;
        }

        $pPath = DataUtil::formatForOS(dirname($path));
        if (FileUtil::mkdirs($pPath, $mode) === false) {
            return false;
        }

        if ($mode) {
            return mkdir($path, $mode);
        }
        else {
            return mkdir($path);
        }
    }


    /**
     * Recursiveley delete given directory path
     *
     * @param path        The path/folder we wish to delete
     *
     * @return boolean The return code from rmdir()
     */
    function deldir ($path)
    {
        $dh = opendir($path);
        $path = DataUtil::formatForOS($path);

        while(($file = readdir($dh)) !== false) {
            $osfile = DataUtil::formatForOS($file);
            if (is_dir("$path/$file") && ($file != '.' && $file != '..')) {
                FileUtil::deldir("${path}/${osfile}");
            }
            else
            if ($file != '.' && $file != '..') {
                unlink("${path}/${osfile}");
            }
        }

        closedir($dh);
        return rmdir($path);
    }


    /**
     * Read a file's contents and return them as a string. This method also
     * opens and closes the file.
     *
     * @param filename    The file to read
     * @param absolute   Allow absolute paths (default=false) (optional)
     *
     * @return string     The file's contents or false upon failure
     */
    function readFile ($filename, $absolute=false)
    {
        if (!strlen($filename)) {
            return pn_exit ('FileUtil::readFile: filename is empty');
        }

        return file_get_contents (DataUtil::formatForOS($filename, $absolute));
    }


    /**
     * Read a file's contents and return them as an array of lines.
     * This method also opens and closes the file.
     *
     * @param filename    The file to read
     * @param absolute   Allow absolute paths (default=false) (optional)
     *
     * @return string     The file's contents or false upon failure
     */
    function readFileLines ($filename, $absolute=false)
    {
        $data  = FileUtil::readFile ($filename, $absolute);
        $lines = explode ("\n", $data);
        return $lines;
    }


    /**
     * Read a serialized's file's contents and return them as a string
     * This method also opens and closes the file.
     *
     * @param filename    The file to read
     * @param absolute   Allow absolute paths (default=false) (optional)
     *
     * @return string     The file's contents or false upon failure
     */
    function readSerializedFile ($filename, $absolute=false)
    {
        return unserialize(FileUtil::readFile ($filename, $absolute));
    }


    /**
     * Take an existing filename and 'randomize' it
     *
     * @param filename   The filename to randomize
     * @param dir        The directory the file should be in
     *
     * @return string    The 'randomized' filename
     */
    function randomizeFilename ($filename, $dir)
    {
        $ext  = '';
        $time = time ();

        if (!$filename) {
            $filename = FileUtil::generateRandomFilename (10, 15, true, true);
        }else
        if (strrchr ($filename, '.') !== false) // do we have an extension?
        {
            $ext = FileUtil::getExtension ($filename);
            $filename = FileUtil::stripExtension ($filename);
        }

        if ($dir) {
            $dir .= '/';
        }

        if ($ext) {
            $rnd = $dir . $filename . '_' . $time . '.' . $ext;
        } else {
            $rnd = $dir . $filename . '_' . $time;
        }

        return $rnd;
    }


    /**
     * Write a string to a file
     * This method also opens and closes the file.
     * On versions >= PHP5 this method will use the file_put_contents API
     *
     * @param filename   The file to write
     * @param data       The data to write to the file
     * @param absolute   Allow absolute paths (default=false) (optional)
     *
     * @return mixed     The return code from the fclose() call
     */
    function writeFile ($filename, $data='', $absolute=false)
    {
        if (!$filename) {
            return pn_exit ('FileUtil::writeFile: filename is empty');
        }

        if (version_compare(phpversion(), '5.0.0', '>='))
            return file_put_contents (DataUtil::formatForOS($filename, $absolute), $data);

        // for PHP4 versions which don't have file_put_contents
        $fd = fopen (DataUtil::formatForOS($filename, $absolute), 'w');
        if ($fd === false) {
            return false;
        }

        fwrite ($fd, $data);
        return fclose ($fd);
    }


    /**
     * Write a serialized string to a file
     * This method also opens and closes the file.
     *
     * @param filename   The file to write
     * @param data       The data to write to the file
     * @param absolute   Allow absolute paths (default=false) (optional)
     *
     * @return mixed     The return code from the fclose() call
     */
    function writeSerializedFile ($filename, $data, $absolute=false)
    {
        return FileUtil::writeFile ($filename, serialize($data), $absolute);
    }


    /**
     * Upload a file
     *
     * @param key           The filename key to use in accessing the file data
     * @param destination   The destination where the file should end up
     * @param newName       The new name to give the file (optional) (default='')
     *
     * @return mixed        The return code from the fclose() call
     */
    function uploadFile ($key, $destination, $newName='')
    {
        if (!$key) {
            return pn_exit ('FileUtil::uploadFile called with invalid field key ...');
        }

        if (!$destination) {
            return pn_exit ('FileUtil::uploadFile called with invalid destination ...');
        }

        $msg = '';
        if ($_FILES[$key]['name']) {
            $uploadfile = $_FILES[$key]['tmp_name'];
            $origfile   = $_FILES[$key]['name'];
            $uploaddest = DataUtil::formatForOS("$destination/$origfile");

            if ($newName) {
                $uploaddest = DataUtil::formatForOS("$destination/$newName");
            }

            $rc = move_uploaded_file($uploadfile, $uploaddest);
            if (!$rc) {
                switch ($_FILES[$key]['error'])
                {
                    case 1:
                        $msg = 'The uploaded file exceeds the upload_max_filesize directive in php.ini.';
                        break;
                    case 2:
                        $msg = 'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the html form.';
                        break;
                    case 3:
                        $msg = 'The uploaded file was only partially uploaded.';
                        break;
                    case 4:
                        $msg = 'No file was uploaded.';
                        break;
                    case 5:
                        $msg = 'Uploaded file size 0 bytes';
                        break;
                }
            }
        }

        return $msg;
    }
}
