<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.search.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display a search form
 *
 * Available parameters:
 *  - active            a comma-separated list of modules to be searched.
 *  - bool              the boolean operation to be performed (AND or OR)
 *  - label             the label to show before the search box
 *  - button            the text to be displayed on the search button
 *  - size              the size of the input box
 *  - value             the default value of the input box
 *  - search_accesskey  the access key of the Search box
 *  - search_tabindex   the tabindex of the Search box (defaults to 0)
 *  - search_class      the CSS class to assign to the Search box
 *  - button_accesskey  the access key of the Search button
 *  - button_class      the CSS class to assign to the Search button
 *  - button_tabindex   the tabindex of the Search button (defaults to 0)
 *  - class             the CSS class to assign to the form
 *  - js                use javascript to automatically clear the default value (defaults to true)
 *
 * Example
 * <!--[search]-->
 *
 * <!--[pnml name="_SEARCH" assign="search_label"]-->
 * <!--[search active="faqs, stories" label=$search_label class="pnsearchform"]-->
 *
 * Note
 * IE (incorrectly) treats a form as a block element rather than an inline
 * element. This, if you want the search box to display as expected on IE,
 * you should use a custom CSS class to style the search box (use the class
 * parameter of this plugin), and add
 *   display: inline;
 * to this class in your style sheet.
 *
 * @author       Mark West
 * @author       Jrg Napp
 * @since        23/10/03
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      the search box
 */
function smarty_function_search($params, &$smarty)
{
    extract($params);
    unset($params);

    // assign the class, if set
    if (isset($class)) {
        $class = 'class="' . DataUtil::formatForDisplay($class) .'"';
    } else {
        $class = '';
    }

    // Staring the search box
    $searchbox  = '<form ' . $class . ' action="'.DataUtil::formatForDisplay(pnModURL('Search', 'user', 'search')).'" method="post">'."\n";
    $searchbox .= ' <div>'."\n";
    $searchbox .= '  <input type="hidden" name="overview" value="1" />'."\n";

    if (!isset($bool) || $bool != 'OR') {
        $bool = 'AND';
    }
    $searchbox .= '  <input type="hidden" name="bool" value="'.DataUtil::formatForDisplay($bool).'" />'."\n";

    // Loop through the active modules and assign them
    if (isset($active)) {
        $active_modules = split(',', $active);
        foreach($active_modules as $active_module) {
            $active_module = trim($active_module);
            $searchbox .= '  <input type="hidden" name="active_'.DataUtil::formatForDisplay($active_module).'" value="1" />'."\n";
        }
	}

    if (isset($label)) {
        $searchbox .= '  <label for="search_plugin_q">'.DataUtil::formatForDisplay($label).'</label>'."\n";
    }
    if (!isset($size)) {
        $size = '12';
    }
    if (!isset($search_tabindex)) {
        $search_tabindex = '0';
    } else {
        $search_tabindex = (int)$search_tabindex;
    }
    if (isset($search_accesskey)) {
        $search_accesskey = 'accesskey="'.DataUtil::formatForDisplay($search_accesskey).'" ';
    } else {
        $search_accesskey = '';
    }
    if (isset($search_class)) {
        $search_class = 'class="'.DataUtil::formatForDisplay($search_class).'" ';
    } else {
        $search_class = '';
    }
    if (!isset($value)) {
        $value = '';
    } else {
        $value = DataUtil::formatForDisplay($value);
    }

    if (!isset($js) || $js) {
        $js = ' onblur="if (this.value==\'\')this.value=\''.$value.'\';" onfocus="if (this.value==\''.$value.'\')this.value=\'\';"';
    } else {
        $js = '';
    }

    $searchbox .= '  <input id="search_plugin_q" type="text" name="q" value="'.$value.'" size="'.DataUtil::formatForDisplay($size).'" tabindex="'.DataUtil::formatForDisplay($search_tabindex).'" '.$search_class.$search_accesskey.$js.' />'."\n";

    if (isset($button)) {
        if (!isset($button_tabindex)) {
            $button_tabindex = '0';
        } else {
            $button_tabindex = (int)$button_tabindex;
        }
        if (isset($button_accesskey)) {
            $button_accesskey = ' accesskey="'.DataUtil::formatForDisplay($button_accesskey).'" ';
        } else {
            $button_accesskey = '';
        }
        if (isset($button_class)) {
            $button_class = ' class="'.DataUtil::formatForDisplay($button_class).'" ';
        } else {
            $button_class = '';
        }
        $searchbox .= '  <input type="submit" value="'.DataUtil::formatForDisplay($button).'" tabindex="'.DataUtil::formatForDisplay($button_tabindex).'"'.$button_accesskey.$button_class.' />'."\n";
    }

    $searchbox .= ' </div>'."\n";
    $searchbox .= '</form>'."\n";

    return $searchbox;
}
